import java.io.*;
import java.util.*;
import java.lang.*;


/** This class represents the global name server. 
  * @author Yin Xiong
  * @author Glenn Matthews
  * @version 1.2 (06/2005)
  */
  
public class GlobalNameServer{


/**A vector that stores the global routing information */
Vector<RoutingInfo> routingTable;

/**Construct a new GlobalNameServer object.*/
public GlobalNameServer()
{
   routingTable=new Vector<RoutingInfo>();
}//end constructor

/**Adds a row of routing inforamtion into the lookup table.
  *@param rinfo an RoutingInfo instance representing a row in the table.
  *@return the size of the lookup table.
  */
public int addRoutingRow(RoutingInfo rinfo)
{
   routingTable.add(rinfo);
   return routingTable.size();
}//end addRoutingRow



/**Looks up the routing info by an LP's application id.
  *@param appid String representing the application id of the LP.
  */
public RoutingInfo getRoutingInfoByAid(String appid)
{
  //System.out.println("in gns, aid: " + appid);
  for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    //System.out.println("in gns, aid, rinfo.aid: " + appid + " " + rinfo.APPid);
    if(appid.equalsIgnoreCase(rinfo.APPid))
 
        return rinfo;
   }//end for
  return null;
    

}//end getReceiverByAid

/**Looks up the routing info by an LP id.
  *@param lid int representing the LP id.
  */
public RoutingInfo getRoutingInfoByLid(int lid)
{
  for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    if(lid==rinfo.LPid)
        return rinfo;
   }//end for
  return null;
}//end getReceiverByLid

/**Displays the contents of the routing table.*/
public void dump()
{
  System.out.println("\nDumping GNS:");
   for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    rinfo.dump();
  }
}//end dump

/**Returns (as a tab-separated list) the APPids of every LP in the application.
 * @since 1.2 */
public String getAPPids() {
	String s = "";
	for (int i=0; i<routingTable.size(); i++) {
		RoutingInfo r = (RoutingInfo)routingTable.get(i);
		s += r.getAPPid() + '\t';
	}
	return s.trim();
}

/**Looks up the PE id of an LP by its LP id.         
  *@param lid int representing the LPid of the LP.               
  */
public int getPidByLid(int lid)     
{
  for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    if(lid==rinfo.getLPid())                    
        return rinfo.getPEid();
   }//end for
  return -4;
}//end getPidByLid



/**Looks up the PE id of an LP by its application id.
  *@param appid String representing the application id of the LP.
  */
public int getPidByAid(String appid)
{
  for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    //System.out.println("appid, thisid: " + appid+ " " + rinfo.getAPPid());

    if(appid.equalsIgnoreCase(rinfo.getAPPid()))
        return rinfo.getPEid();
   }//end for
  return -2;
}//end getPidByAid

/**Looks up the LP id of an LP by its application id.
  *@param appid String representing the application id of the LP.
  */
public int getLidByAid(String appid)
{
  for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);

//System.out.println("in gns,APPid: " + rinfo.getAPPid());
    if(appid.equalsIgnoreCase(rinfo.getAPPid()))
        return rinfo.getLPid();
   }//end for
  return -3;
}//end getLidByAid

}//end class GlobalNameServer
