
import java.io.*;
import java.util.*;
import java.lang.*;
import java.net.*;
import java.lang.Thread;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/** This class is the API for simulation application programmer. It receives commandline
  * parameters, maps LPs to PES, initiates, runs and terminates the simulation.
  * @author Yin Xiong
  * @version 1.7 (09/2006)
  */
  
 
public class Sim{
/**The main function that runs the simulation. 
  */

static public void main(String[] args) {
    MessagePool msgPool=new MessagePool();
    GlobalNameServer gns=new GlobalNameServer();
    Flag eosFlag=new Flag();
    boolean endOfSim=false;
    Flag gvtFlag=new Flag();
    int gvtClock=5;

    PE myPE=null;
    int sleepTime=300;
    int lpForMe=1;    //how many LPs each PE will have except PE0
    int lpForFirst=0; //how many LPs the first PE will have
    int lpForEach=0;  //how many LPs each of  the rest PE will have
    int lpTotal=0;    //how many LPs altogether
    String myname=null;
	
	/**a vector of LP configuration information*/
	Vector lpConfigInfoVector=new Vector();
	
	String fileName;
	int numPEs;
	Creator creator;
    
    int len,tasks,me,size,rc;
	me=size=0;
 
   
	
//get file name from command line
if(args.length<4){
  System.out.println("Usage: java Sim [config file name] [# of PEs][sleep time][GVT clock]");
  System.exit(1);
}

else{
  fileName=args[0];
  //System.out.println("Config file: " +fileName + "\n");
	
  numPEs=Integer.parseInt(args[1]);
  //System.out.println("Number of PEs: " + numPEs+ "\n");

  sleepTime=Integer.parseInt(args[2]);
  //System.out.println("sleep time: " + sleepTime);

  gvtClock=Integer.parseInt(args[3]);
  System.out.println("GVT will be computed every " + gvtClock + " sim time");

  //invoke Creator to parse config file 
  creator=new Creator();
  
  //process config file
  creator.procConfigFile(fileName, numPEs);
  
  //get LP info: how many LPs for each PE and the info for each LP  
   lpConfigInfoVector=creator.getLPConfigInfo();  
     
  //for debugging only: check if LP config info is obtained correctly
  for(int i=0;i<lpConfigInfoVector.size();i++){
     LPConfigInfo lpinfo=(LPConfigInfo)lpConfigInfoVector.get(i);
	 //System.out.println(lpinfo.toString());
  }
  
  //gets the num of LPs for the first PE, which can be different from the rest
  lpForFirst=creator.getLPNumForFirst();
  
  //gets the number of LPs for each of the rest PEs
  lpForEach=creator.getLPNumForEach();
  
  System.out.println("LP num for the first PE and each of the rest: " + lpForFirst + " " + lpForEach);
   
   //create PEs (as threads) and send each PE's LP info to the PE
  int numThreads=numPEs-1;
  int numLPs=0;
  int first=0;
  int last=first+lpForFirst;
  PE PEs[]=new PE[numThreads];
  Thread peThreads[]=new Thread[numThreads];
  
  //create PEs and each PE create LPs
  for(int i=0;i<numThreads;i++){
    PEs[i]=new PE(i,msgPool,eosFlag,gvtFlag, gvtClock);
	LocalNameServer mylns=new LocalNameServer(i);
        PEs[i].setSleepTime(sleepTime);
	
	System.out.println("\nPE,first,last: " + i + " " + first + " " +last);
	for(int j=first;j<last;j++){
	   LPConfigInfo lpConfigInfo=(LPConfigInfo)lpConfigInfoVector.get(j);
	   String id=lpConfigInfo.getLPIndex();
	   String objname=lpConfigInfo.getLPName();
	   String classname=lpConfigInfo.getLPClassName();
	   String dat=lpConfigInfo.getLPData();;
       
	   //PEid,LPid,Appid,APPname,classname,data                           
	   PEs[i].createLPs(i,j, id, objname, classname, dat);
	   System.out.println("Pid,Lid,Aid,Aname,Cname,data: " +i + " " + j + " " +id + " " +objname + " " + classname + " " + dat);
	   
	   //add routing info to LNS and GNS
	   RoutingInfo rinfo=new RoutingInfo(i,j,id);
	   mylns.addRoutingRow(rinfo);
	   gns.addRoutingRow(rinfo);
	   
	   //System.out.println(lpConfigInfo.toString());
	}//for j
	
	//set this PE's LNS
	PEs[i].setLNS(mylns);
	
	//set this PE's GNS
	PEs[i].setGNS(gns);
	
	//for debugging only: dump LNS
	PEs[i].dumpLNS();
	
	
	//updates first and last 
	first=last;
	last=last+lpForEach;
}//end for i

  //for debugging only: dump GNS
  //gns.dump();
  
  //run PEs

      // create new thread pool with two threads
      ExecutorService application = Executors.newFixedThreadPool( numThreads+1 );
 try
      {
      //set up a socket server for monitoring/steering application
      //SimSocket socketServer=new SimSocket(gns, msgPool, flag);
      //application.execute(socketServer);

      Controller gnsServer=new Controller(PEs,eosFlag,gvtFlag);
      application.execute(gnsServer);

      for(int i=0;i<numThreads;i++){  
       application.execute(PEs[i]);
       }
       
      } // end try
      catch ( Exception exception )
      {
         exception.printStackTrace();
      } // end catch

//  System.out.println("before shutdown");

      application.shutdown(); // terminate application when threads end

  //System.out.println("after exercuter shutdown");

 }//end else 
}//end main


}//end class Sim


